"""
A weird tool for making models from mask images. The point is to be able to
create bases of models with image manipulation programs. The resulting models
can then be taken to voxel editor for further editing.

The usage is a bit hard, but here are some tips. This file contains two tools:
one for creating mask images and one for combining the images into a voxel
file. The images are simple black and white masks and each shows the model
from different direction as a silhouette. For example xy means x is the
horizontal axis pointing right and y is the vertical axis pointing up.

Example:
You can actually make the images on your own, too, but the create tool is
there to help the process.

python mask_tool.py create test 2 5 7

This line creates images test_xy.png, text_xz.png and test_yz.png that all are
fully white. The model will be 2x5x7 (x,y,z) in size. Now these images can be
edited. Next step will be making a voxel model of the edited images with the
following line:

python mask_tool.py combine test model.vox

This takes all the three images and creates a model file called model.vox.

I admit this is a bit confusing. :P
"""

import sys
import argparse
from model import Model
try:
	import Image
except ImportError:
	print "Error! PIL library is needed!"
	sys.exit(-1)

def create_base_images(filename, x, y, z):
	if x <= 0 or y <= 0 or z <= 0:
		raise ValueError("Dimensions must be positive")
	size = (x,y,z)
	filetype = "png"

	filestring = "{}_{{}}.{}".format(filename, filetype)
	mask_bit = 1

	im_xy = Image.new("1", (size[0], size[1]), mask_bit)
	im_xy.save(filestring.format("xy"))
	im_xz = Image.new("1", (size[0], size[2]), mask_bit)
	im_xz.save(filestring.format("xz"))
	im_yz = Image.new("1", (size[1], size[2]), mask_bit)
	im_yz.save(filestring.format("yz"))

def combine(filename):
	filetype = "png"
	# extract base name of file ("mask_xy.png" => "mask")
	for axes in ("xy","xz","yz"):
		if filename.endswith("_{}.{}".format(axes, filetype)):
			filename = filename[:-7] # remove ending
	filestring = "{}_{{}}.{}".format(filename, filetype)

	images = {}
	for axes in ("xy","xz","yz"):
		images[axes] = Image.open(filestring.format(axes))

	if not validate_dimensions(images):
		raise ValueError("Dimensions of the images didn't match")

	size_x = images["xy"].size[0]
	size_y = images["xy"].size[1]
	size_z = images["xz"].size[1]

	# load pixel data for accessing points
	pixels = {axes:image.load() for (axes, image) in images.items()}


	# create the model
	model = Model(size_x, size_y, size_z)
	model.set_offsets(-size_x/2, -size_y/2, -size_z/2)
	for x in xrange(size_x):
		for y in xrange(size_y):
			for z in xrange(size_z):
				color_index = 0
				mask = get_mask_bit(pixels["xy"], x, size_y-1-y) and\
					get_mask_bit(pixels["xz"], x, size_z-1-z) and\
					get_mask_bit(pixels["yz"], y, size_z-1-z)

				model.set_block(x, y, z, color_index if mask else Model.empty_index)

	return model

def validate_dimensions(images):
	if images["xy"].size[0] == images["xz"].size[0] and\
			images["xy"].size[1] == images["yz"].size[0] and\
			images["xz"].size[1] == images["yz"].size[1]:
		return True
	return False

def get_mask_bit(pixel_data, a, b):
	pixel = pixel_data[a, b]
	if isinstance(pixel, int):
		return bool(pixel)
	# RGB tuple:
	return bool(pixel[0])


def combine_args(args):
	model = combine(args.input_file)
	with open(args.output_file, "wb") as fp:
		model.save(fp)

def create_args(args):
	create_base_images(args.output_file, args.x, args.y, args.z)


def main():
	# Parse arguments:
	parser = argparse.ArgumentParser()
	subparsers = parser.add_subparsers()

	parser_create = subparsers.add_parser("create",
		description="Creates three images corresponding to the given dimensions")
	parser_create.add_argument("output_file", type=str,
		help="base name of the resulting model file. names will be ")
	parser_create.add_argument("x", type=int,
		help="x size of the model")
	parser_create.add_argument("y", type=int,
		help="y size of the model")
	parser_create.add_argument("z", type=int,
		help="z size of the model")
	parser_create.set_defaults(func=create_args)

	parser_combine = subparsers.add_parser("combine",
		description="Creates a model out of three mask images on dimensions xy, xz, yz")
	parser_combine.add_argument("input_file", type=str,
		help="base name of the input image files")
	parser_combine.add_argument("output_file", type=str,
		help="name of the resulting model file")
	parser_combine.set_defaults(func=combine_args)

	args = parser.parse_args()
	args.func(args)



if __name__ == '__main__':
	main()
